@@ -13,9 +13,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Written by:
  *     Jasper St. Pierre <jstpierre@mecheye.net>
@@ -48,7 +46,6 @@ struct _GisPage
   GtkBin parent;
 
   GisDriver *driver;
-  GtkBuilder *builder;
 
   GisAssistantPagePrivate *assistant_priv;
 };
@@ -58,7 +55,6 @@ struct _GisPageClass
   GtkBinClass parent_class;
   char *page_id;
 
-  GtkBuilder * (*get_builder) (GisPage *page);
   void         (*locale_changed) (GisPage *page);
   gboolean     (*apply) (GisPage *page,
                          GCancellable *cancellable);
@@ -72,6 +68,10 @@ char *       gis_page_get_title (GisPage *page);
 void         gis_page_set_title (GisPage *page, char *title);
 gboolean     gis_page_get_complete (GisPage *page);
 void         gis_page_set_complete (GisPage *page, gboolean complete);
+gboolean     gis_page_get_skippable (GisPage *page);
+void         gis_page_set_skippable (GisPage *page, gboolean skippable);
+gboolean     gis_page_get_needs_accept (GisPage *page);
+void         gis_page_set_needs_accept (GisPage *page, gboolean needs_accept);
 GtkWidget *  gis_page_get_action_widget (GisPage *page);
 void         gis_page_locale_changed (GisPage *page);
 void         gis_page_apply_begin (GisPage *page, GisPageApplyCallback callback, gpointer user_data);
